# ZPyIrc
#
# Copyright (c) 2003 by Sylvain Hellegouarch.
#
# This software is released under the terms of the GNU General
# Public License version 2, or (at your option) any later
# version. It is distributed WITHOUT ANY WARRANTY. See the GNU
# General Public License for more details.
#

#Zope imports
from Globals import InitializeClass
from Globals import package_home
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from OFS.Folder import Folder
from OFS.Image import manage_addImage
import zLOG
from AccessControl import ClassSecurityInfo
from Products.PythonScripts.Utility import allow_module
allow_module("zpyircconnexion")

#Python imports
import re
import md5
from os.path import join
import string
import dircache
from time import time

#Internal product import
import ZPyIrcConnexion

_zpyirc_version = "0.1.3"
   
manage_addZPyIrcForm = PageTemplateFile('ui/ZPyIrc_add.pt', globals())
def manage_addZPyIrc(self, id, title, default_server,default_port,default_channel,default_nickname,default_username,max_connexions,timeout,select_timeout,default_password,default_host,default_format,default_css_style,can_change_nickname='off',can_change_server='off',can_change_channel='off',can_change_port='off',can_change_username='off',REQUEST=None):
    """Add a ZPyIrc object"""
    self._setObject(id, ZPyIrc(id, title,default_server,can_change_server,default_port,can_change_port,default_channel,can_change_channel,default_nickname,can_change_nickname,default_username,can_change_username,max_connexions,timeout,select_timeout,default_password,default_host,default_format,default_css_style))
    if REQUEST is not None:
      return self.manage_main(self, REQUEST)

class ZPyIrc(Folder):
    meta_type = 'ZPyIrc'
    security = ClassSecurityInfo()
    security.declareObjectProtected('View')
      
    def get_css_styles():
        """Read the css directory for various css styles"""
        css_styles = {}
        try:
            # join method from os.path module
            # package_home from Globals module
            products_path = join(package_home(globals()))
            # What css sheets do we have ?
            css_path = join(products_path,'css')
            css_sheets = []
            css_sheets = dircache.listdir(css_path)
            for css in css_sheets:
                css_key = css[:-4]
                css_styles[css_key] = PageTemplateFile(join(css_path,css), globals())
        except:
            zLOG.LOG("ZPyIrcDebug : ",0,"Error while reading css directory")
        return css_styles
    
    #view = PageTemplateFile('ui/ZPyIrc_view.pt', globals())
    index_html = PageTemplateFile('ui/ZPyIrc_index_html.pt', globals())
    edit = PageTemplateFile('ui/ZPyIrc_edit.pt', globals())
    max_connexion = PageTemplateFile('ui/ZPyIrc_max.pt', globals())
    sweep = PageTemplateFile('ui/ZPyIrc_sweep.pt', globals())
    error = PageTemplateFile('ui/ZPyIrc_error.pt', globals())
    info = PageTemplateFile('ui/ZPyIrc_info.pt', globals())
    
    css_styles = get_css_styles()
    
    __ac_permissions__=(
        ('Change ZPyIrc', ('edit','sweep','info')),
    )

    security.setPermissionDefault('Change ZPyIrc', ('Manager', 'ZPyIrc Owner'))
     
    def __init__(self, id, title,default_server,can_change_server,default_port,can_change_port,default_channel,can_change_channel,default_nickname,can_change_nickname,default_username,can_change_username,max_connexions,timeout,select_timeout,default_password,default_host,default_format,default_css_style):
        """ZPyIrc class"""
        self.id = id    
        self.title = title
        self.version = _zpyirc_version
        self.default_server = default_server
        self.default_port = default_port
        self.default_channel = default_channel
        self.default_nickname = default_nickname
        self.default_username = default_username
        self.default_password = default_password
        self.default_host = default_host
        self.can_change_server = can_change_server
        self.can_change_port = can_change_port
        self.can_change_channel = can_change_channel
        self.can_change_nickname = can_change_nickname
        self.can_change_username = can_change_username
        self.max_connexions = int(max_connexions)
        self.nb_connexion = 0
        self.timeout = int(timeout)
        self.select_timeout = float(select_timeout)
        self.error_message = ""
        self.formats = {}
        self.default_format = default_format
        self.default_css_style = default_css_style
        self.read_format_files()
        self.manage_addFolder('images','images')
        self.read_images_dir()
        self.help_data = ""
        for line in open(join(join(join(package_home(globals())),'docs'),'help.html'),'r+b').readlines():
            if line != "\n":
                self.help_data = self.help_data + string.replace(line[:-1],"'","\\'")
            
    def read_images_dir(self):
        """Read all images"""
        try:
            # join method from os.path module
            # package_home from Globals module
            products_path = join(package_home(globals()))
            # What formats do we have ?
            images_path = join(products_path,'images')
            images_files = []
            images_files = dircache.listdir(images_path)
            images_folder = getattr(self,'images')
            for image in images_files:
                images_folder.manage_addImage(id=image,file=open(join(images_path,image),'r+b'))
        except:
            zLOG.LOG("ZPyIrcDebug : ",0,"Error while reading images files")
            
    def read_format_files(self):
        """Read all format files from the formats directory"""
        try:
            # join method from os.path module
            # package_home from Globals module
            products_path = join(package_home(globals()))
            # What formats do we have ?
            formats_path = join(products_path,'formats')
            format_files = []
            format_files = dircache.listdir(formats_path)
            for fname in format_files:
                fd = open(join(formats_path,fname),'r+b')
                lines = fd.readlines()
                fd.close()
                self.formats[fname] = {}
                format_targets = {}
                fmts = {}
                for line in lines:
                    if line not in ["\n","\r\n"] and line[0] != '#':
                        cmd, desc = string.split(line,'=',1)
                        cmd = string.strip(cmd)
                        desc = string.strip(desc)
                        if line[0] != '.':
                            format_targets[cmd] = desc
                        else:
                            fmts[cmd] = desc
                targets = {}
                for cmd, desc in format_targets.items():
                    for mapping_key,mapping in fmts.items():
                        if string.find(desc,mapping_key) != -1:
                            targets[cmd] = desc % fmts
                self.formats[fname] = targets
        except Exception,s :
            zLOG.LOG("ZPyIrcDebug : ",0,"Error while reading format files on " + str(s))
        except:
            zLOG.LOG("ZPyIrcDebug : ",0,"Error while reading format files")
        
    def manage_beforeDelete(self,item,container):
        """Here we want to close correctly each connection still alive"""
        for id in self.objectIds('ZPyIrcConnexion'):
            self.manage_delObjects(id)

    def get_number_of_users(self):
        """Get the number of users active or not within the client"""
        return len(self.objectIds())

    def get_active_connexions(self):
        """Get the number of active connections"""
        count = 0
        for client in self.objectValues('ZPyIrcConnexion'):
            if client.is_alive():
                count = count + 1
        return count

    def do_sweep(self,REQUEST=None):
        """Clean all unused client item object"""
        for id,client in self.objectItems('ZPyIrcConnexion'):
            if not client.is_alive():
                self.manage_delObjects(id)
                self.nb_connexion = self.nb_connexion - 1
            else:
                if (time() - client.idle_duration) > self.timeout:
                    self.manage_delObjects(id)
                    self.nb_connexion = self.nb_connexion - 1
        return self.REQUEST.RESPONSE.redirect('index_html')

    security.declarePublic('get_error_message')
    def get_error_message(self):
        """Get the last error message"""
        return self.error_message
    
    security.declarePublic('manage_addZPyIrcConnexion')
    def manage_addZPyIrcConnexion(self,REQUEST=None):
        """Add a user client"""
        if self.max_connexions != -1:
            if self.max_connexions == self.nb_connexion:
                return self.REQUEST.RESPONSE.redirect('max_connexion')
        self.nb_connexion = self.nb_connexion + 1
        r = self.REQUEST
        server = self.default_server
        port = self.default_port
        channel = self.default_channel
        nickname = self.default_nickname
        username = self.default_username
        realname = ""
        format = self.default_format
        self.error_message = ""
        
        #Let's make some tests to verify submitted data... better to handle errors now than later
        if self.can_change_server == 'on':
            if r['zpyirc_server_name'] == '':
                self.error_message = "You must provide a server name or address"
                return self.REQUEST.RESPONSE.redirect('error')
            else:
                server = r['zpyirc_server_name']
        if self.can_change_port == 'on':
            if r['zpyirc_port'] == '':
                self.error_message = "You must provide a port"
                return self.REQUEST.RESPONSE.redirect('error')
            else:
                port = r['zpyirc_port']
        if self.can_change_channel == 'on':
            if r['zpyirc_channel'] == '':
                self.error_message = "You must provide a channel"
                return self.REQUEST.RESPONSE.redirect('error')
            else:
                channel = r['zpyirc_channel']
        if self.can_change_channel == 'on':
            if re.compile(r"^#[\w]*$").match(r['zpyirc_channel']) == None:
                self.error_message = "The channel your provided is not correct"
                return self.REQUEST.RESPONSE.redirect('error')
            else:
                channel = r['zpyirc_channel']
        if self.can_change_nickname == 'on':
            if r['zpyirc_nickname'] == '':
                self.error_message = "You must provide a nickname"
                return self.REQUEST.RESPONSE.redirect('error')
            else:
                nickname = r['zpyirc_nickname']
        if self.can_change_username == 'on':
            if r['zpyirc_username'] == '':
                self.error_message = "You must provide a username"
                return self.REQUEST.RESPONSE.redirect('error')
            else:
                username = r['zpyirc_username']
        if r['zpyirc_realname'] == '':
            self.error_message = "You must provide a real name"
            return self.REQUEST.RESPONSE.redirect('error')
        else:
            realname = r['zpyirc_realname']

        format = r['zpyirc_format']
        css_style = ZPyIrc.css_styles[r['zpyirc_css_style']]

        id = md5.new(str(nickname + " " + username + " " + realname)).hexdigest()
        #Here we check the previous existence of the object with this id, if it exists already we just have to restart a new new connexion
        if id not in self.objectIds():
            ZPyIrcConnexion.manage_addZPyIrcConnexion(self,id,server,port,channel,nickname,username,realname,self.formats,format,css_style,self.default_password,self.default_host)
        else:
            obj = getattr(self,id)
            if not obj.is_alive():
                obj.restart_connexion(server,port,channel,format,css_style)
        return self.REQUEST.RESPONSE.redirect(id)

    def update_client(self):
        """Update the client data"""
        self.title = self.REQUEST['title']
        self.default_server = self.REQUEST['default_server']
        self.default_port = self.REQUEST['default_port']
        self.default_channel = self.REQUEST['default_channel']
        self.default_nickname = self.REQUEST['default_nickname']
        self.default_username = self.REQUEST['default_username']
        self.default_password = self.REQUEST['default_password']
        self.default_host = self.REQUEST['default_host']
        
        if self.REQUEST.has_key('can_change_server'):
            self.can_change_server = 'on'
        else:
            self.can_change_server = 'off'
        if self.REQUEST.has_key('can_change_port'):
            self.can_change_port = 'on'
        else:
            self.can_change_port = 'off'
        if self.REQUEST.has_key('can_change_channel'):
            self.can_change_channel = 'on'
        else:
            self.can_change_channel = 'off'
        if self.REQUEST.has_key('can_change_nickname'):
            self.can_change_nickname = 'on'
        else:
            self.can_change_nickname = 'off'
        if self.REQUEST.has_key('can_change_username'):
            self.can_change_username = 'on'
        else:
            self.can_change_username = 'off'
        self.max_connexions = int(self.REQUEST['max_connexions'])
        self.timeout = int(self.REQUEST['timeout'])
        self.select_timeout = float(self.REQUEST['select_timeout'])
        self.default_format = self.REQUEST['default_format']
        self.default_css_style = self.REQUEST['default_css_style']
        
        return self.REQUEST.RESPONSE.redirect(self.id)

    def delete_client(self,id):
        self.manage_delObjects(id)

InitializeClass(ZPyIrc)
