# ZPyIrc
#
# Copyright (c) 2003 by Sylvain Hellegouarch.
#
# This software is released under the terms of the GNU General
# Public License version 2, or (at your option) any later
# version. It is distributed WITHOUT ANY WARRANTY. See the GNU
# General Public License for more details.
#

#Zope specific import
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from ImageFile import ImageFile
from Acquisition import Implicit
from Globals import Persistent
from OFS.SimpleItem import Item
from Products.PythonScripts.Utility import allow_module
allow_module('pyirc')
import zLOG

#Python specific import
import string
import select
import types
from socket import gethostbyname
from re import compile,findall,sub,escape
from time import strftime,gmtime,time

#Let's import the core module
import pyirc

#Manage connexions
_pyirc_objects = {}

_html_colors_reg = compile("([\w|\s]*)(\#\w{6})([\w|\s]*)")
_motd_spaces_to_nbsp = compile("(?<=\s)\s")
_url_finder = compile("((?:http|ftp)://www\.(?:[^ \s<\)]+))")

def manage_addZPyIrcConnexion(self,id,server,port,channel,nickname,username,realname,formats,default_format,default_css_style,password,host):
    """Add ZPyIrcConnexion"""
    self._setObject(id, ZPyIrcConnexion(id,id,server,port,channel,nickname,username,realname,formats,default_format,default_css_style,password,host))

class ZPyIrcConnexion(Implicit, Persistent, Item):
    meta_type = "ZPyIrcConnexion"
    
    # UI taken from CGI:Irc product
    cindex = PageTemplateFile('ui/ZPyIrcConnexionInterface_cindex.pt', globals())
    index_html = PageTemplateFile('ui/ZPyIrcConnexionInterface_cview.pt', globals())
    cform = PageTemplateFile('ui/ZPyIrcConnexionInterface_cform.pt', globals())
    cwindowlist = PageTemplateFile('ui/ZPyIrcConnexionInterface_cwindowlist.pt', globals())
    cmain = PageTemplateFile('ui/ZPyIrcConnexionInterface_cmain.pt', globals())
    cuserlist = PageTemplateFile('ui/ZPyIrcConnexionInterface_cuserlist.pt', globals())
    style_default = None
    
    def __init__(self,id,title,server,port,channel,nickname,username,realname,formats,default_format,default_css_style,password,host):
        """ZPyIrcConnexion class"""
        self.id = id
        self.title = title
        self.port = port
        self.server = server
        self.nickname = nickname
        self.username = username
        self.realname = realname
        self.password = password
        self.host = host
        self.queries = []
        self.formats = formats
        self.default_format = None
        self.away = 0
        self.idle_duration = 0
        self.help_asked = 0
        try:
            self.restart_connexion(server,port,channel,default_format,default_css_style)
        except RuntimeError, s :
            raise RuntimeError, s
        
    def restart_connexion(self,server,port,channel,format,default_css_style):
        """Restart a connexion"""
        self.port = port
        self.server = server
        ZPyIrcConnexion.style_default = default_css_style
        self.away = 0
        self.idle_duration = 0
        self.help_asked = 0
        self.queries = []
        self.idle_duration = time()
        self.default_format = self.formats[format]
        connexion = pyirc.IRCServerConnexion(server,port,self.nickname,self.username,self.realname,self.password,self.host)
        try:
            connexion.connect()
            connexion.join(channel)
            _pyirc_objects[self.id] = connexion
        except:
            raise RuntimeError , str("Can't connect to " + server + " on port " + str(port))
        
    def manage_beforeDelete(self,item,container):
        """Tell the server you're leaving and close the socket connexion nicely"""
        if self.id in _pyirc_objects.keys():
            if self.is_alive():
                #self.REQUEST.RESPONSE.write('<script language="JavaScript" type="text/JavaScript">parent.set_connected(0);</script>\r\n')
                #html = self.default_format['irc close']
                #data = "'-all',"+str("'<span>"+html+"</span><br>'")+",1,1"
                #self._out('witemaddtext',data)
                _pyirc_objects[self.id].quit()
                _pyirc_objects[self.id].disconnect()
            del _pyirc_objects[self.id]
            self._p_changed = 1

    def is_alive(self):
        """Return true if the connexion is still up"""
        if self.id not in _pyirc_objects.keys():
            return 0
        return _pyirc_objects[self.id].is_alive()
    
    def _send_common_message(self,target,message):
        """Send a simple message (not a command) to the active channel"""
        html = self.default_format['message public own'] % (self.nickname,self._escape(message))
        data = "'"+target+"',"+str("'<span>"+html+"</span><br>'")+",1,1" 
        self._out('witemaddtext',data)
        _pyirc_objects[self.id].privmsg(target,message)

    def _send_join(self,channel):
        """Join a new channel"""
        _pyirc_objects[self.id].join(channel)

    def _send_part(self,channel):
        """Leave a channel or a user query"""
        if channel[0] in ['#','&','!']:
            _pyirc_objects[self.id].part(channel)

    def _send_nick(self,nick):
        """Change your nickname"""
        self._out('mynick',"'"+nick+"'")
        _pyirc_objects[self.id].nick(nick)
        
    def _send_query(self,who,message):
        """Send a query to user"""
        self._out('witemadd',"'"+who+"',1")
        self.queries.append(who)
        _pyirc_objects[self.id].privmsg(who,message)
        
    def _send_ctcp_action(self,target,message):
        """Send a /me message"""
        html = self.default_format['action public own'] % (self.nickname,message)
        data = "'"+target+"',"+str("'<span>"+html+"</span><br>'")+",1,1" 
        self._out('witemaddtext',data)
        _pyirc_objects[self.id].ctcp_action(target,message)

    def _send_away(self,message=""):
        """Tell the server you're away"""
        _pyirc_objects[self.id].away(message)

    def _send_back(self):
        """Tell the server you're back"""
        _pyirc_objects[self.id].away()

    def _send_topic(self,target,topic=""):
        """Set/Ask the topic of the current channel"""
        _pyirc_objects[self.id].topic(target,topic)

    def _send_userhost(self,user):
        """Ask user host information"""
        _pyirc_objects[self.id].userhost(user)

    def _send_kick(self,target,user,comment=""):
        """Kick an user from the current channel"""
        _pyirc_objects[self.id].kick(target,user,comment)

    def _send_voice(self,target,user):
        """Voice an user"""
        _pyirc_objects[self.id].mode_channel(target,"+v",user)
    
    def _send_unvoice(self,target,user):
        """Unvoice an user"""
        _pyirc_objects[self.id].mode_channel(target,"-v",user)
    
    def _send_op(self,target,user):
        """Op an user"""
        _pyirc_objects[self.id].mode_channel(target,"+o",user)
    
    def _send_deop(self,target,user):
        """Deop an user"""
        _pyirc_objects[self.id].mode_channel(target,"-o",user)

    def _send_invite(self,user,channel):
        """Invite user to channel"""
        _pyirc_objects[self.id].invite(user,channel)

    def _send_whois(self,user):
        """Send a WHOIS command"""
        _pyirc_objects[self.id].whois(user)
        
    def _missing_parameters(self,target):
        """Tell the user that he/she didn't pass enough paramaters to the command he requested"""
        html = self.default_format['command notparams']
        data = "'"+target+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
        self._out('witemaddtext',data)
        
    def say_something(self,REQUEST=None):
        """Say something to the server"""
        self.idle_duration = time()
        r = self.REQUEST
        cmd = ""
        target = ""
        data = ""
        if r.has_key('target'):
            target = r['target']
        if r.has_key('say'):
            data = r['say']
        
        #zLOG.LOG("ZPyIrcDebug : ",0,data)
        
        if data != "" and data[0] != "/":
            self._send_common_message(target,data)
        else:
            tmp = string.split(data,' ',1)
            cmd = tmp[0]
            if len(tmp) == 2:
                data = tmp[1]
            else:
                data = ""
            if cmd == "/join":
                if data != "":
                    channels = string.split(data,' ')
                    self._send_join(channels[0])
                else:
                    self._missing_parameters(target)
            elif cmd == "/part":
                if data == "":
                    self._send_part(target)
                else:
                    channels = string.split(data,' ')
                    self._send_part(channels[0])
            elif cmd == "/nick":
                if data != "":
                    self._send_nick(data)
                else:
                    self._missing_parameters(target)
            elif cmd == "/msg":
                if data != "":
                    d = string.split(data,' ',1)
                    if len(d) == 2:
                        self._send_query(d[0],d[1])
                    else:
                        self._send_query(d[0],str(d[0]+"?"))
                else:
                    self._missing_parameters(target)
            elif cmd == "/me":
                if data != "":
                    self._send_ctcp_action(target,data)
                else:
                    self._missing_parameters(target)
            elif cmd == "/away":
                if self.away == 0:
                    self.away = 1
                    if data == "":
                        self._send_away()
                    else:
                        self._send_away(data)
                else:
                    self.away = 0
                    self._send_back()
            elif cmd == "/topic":
                if data != "":
                    self._send_topic(target,data)
                else:
                    self._send_topic(target)
            elif cmd == "/userhost":
                if data != "":
                    self._send_userhost(data)
                else:
                    self._missing_parameters(target)
            elif cmd == "/kick":
                if data != "":
                    user_and_comment = string.split(data,' ',1)
                    if len(user_and_comment) == 2:
                        self._send_kick(target,user_and_comment[0],user_and_comment[1])
                    else:
                        self._send_kick(target,user_and_comment[0])
                else:
                    self._missing_parameters(target)
            elif cmd == "/voice":
                if data != "":
                    self._send_voice(target,data)
                else:
                    self._missing_parameters(target)
            elif cmd == "/unvoice":
                if data != "":
                    self._send_unvoice(target,data)
                else:
                    self._missing_parameters(target)
            elif cmd == "/op":
                if data != "":
                    self._send_op(target,data)
                else:
                    self._missing_parameters(target)
            elif cmd == "/deop":
                if data != "":
                    self._send_deop(target,data)
                else:
                    self._missing_parameters(target)
            elif cmd == "/invite":
                if data != "":
                    tmp = string.split(data,' ')
                    if len(tmp) == 2:
                        self._send_invite(tmp[0],tmp[1])
                    else:
                        self._send_invite(tmp[0],target)
            elif cmd == "/whois":
                if data != "":
                    self._send_whois(data)
                else:
                    self._missing_parameters(target)
            elif cmd == "/help":
                self._out('witemadd',"'Help',0")
                data = "'Help','<span>"+self.aq_parent.help_data+"</span>',1,1"
                self._out('witemaddtext',data)
            elif cmd == "/winclose":
                if data != "":
                    self._send_part(data)
                    self._out('witemdel',"'"+data+"'")
                else:
                    self._send_part(target)
                    self._out('witemdel',"'"+target+"'")
            elif cmd == "/quit":
                self.REQUEST.RESPONSE.write('<script language="JavaScript" type="text/JavaScript">parent.set_connected(0);</script>\r\n')
                html = self.default_format['irc close']
                _data = "'-all',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',_data)
                _pyirc_objects[self.id].quit()
                _pyirc_objects[self.id].disconnect()
        self._p_changed = 1

    # It might sound stupid that each client owns its own select loop... could be changed in future release
    #Still need to evaluate the cost of such a loop
    def handle_incoming(self,REQUEST=None):
        """Handle incoming data"""
        self.REQUEST.RESPONSE.setStatus('OK')
        self.REQUEST.RESPONSE.addHeader('Content-Type', 'text/html')
        self.REQUEST.RESPONSE.write('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">\r\n<html>\r\n<head></head>\r\n<body>\r\n')
        self._out('witemadd',"'Status',0")
        self._out('witemnospeak', "'Status'")
        html = self.default_format['zpyirc welcome'] % self.aq_parent.version
        data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
        self._out('witemaddtext',data)
        html = self.default_format['looking up'] % self.server
        data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
        self._out('witemaddtext',data)
        html = self.default_format['connecting'] % (self.server,gethostbyname(self.host),str(self.port))
        data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
        self._out('witemaddtext',data)
        
        sock = []
        sock.append(_pyirc_objects[self.id].get_socket())
        has_quit_on_error = 0
        session_timeout = 0
        while self.is_alive() and not has_quit_on_error and not session_timeout:

            if (time() - self.idle_duration) > self.aq_parent.timeout:
                html = self.default_format['session timeout']
                data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
                session_timeout = 1
                
            try:
                inc,out,err = select.select(sock,[],[],self.aq_parent.select_timeout)
                if len(inc) != 0:
                    self._get_next_message()
                elif len(err) != 0:
                    has_quit_on_error = 1
            except:
                has_quit_on_error = 1

        if has_quit_on_error:
            if self.is_alive():
                _pyirc_objects[self.id].quit()
                _pyirc_objects[self.id].disconnect()
          
        self.REQUEST.RESPONSE.write('<script language="JavaScript" type="text/JavaScript">parent.set_connected(0);</script>\r\n')
          
        self.REQUEST.RESPONSE.write('</body></head>\r\n')
        
    def _escapejs(self,line):
        line = string.replace(line,"\\","\\\\")
        line = string.replace(line,"'","\\'")
        line = string.replace(line,"</script","<\\/\\script")
        return line

    def _escapehtml(self,line):
        line = string.replace(line,"<","&lt;")
        line = string.replace(line,">","&gt;")
        line = string.replace(line,"\"","&quot;")
        return line

    def _escape(self,line):
        # Let's do some escaping...
        line = self._escapejs(line)
        line = self._escapehtml(line)
        return line
    
    def _out(self,func,data):
        # Make any links clickable :)
        data = sub(_url_finder,'<a href="\\1" target="_blank">\\1</a>\\2',data)
        self.REQUEST.RESPONSE.write('<script language="JavaScript" type="text/JavaScript"> parent.'+func+'('+data+');</script>\r\n')
        
    def _get_next_message(self):
        """Get the next message queuing"""
        _pyirc_objects[self.id].process()
        next_msg = []
        text = ""
        while _pyirc_objects[self.id].has_more_messages():
            msg = _pyirc_objects[self.id].get_next_message()
            #zLOG.LOG("ZPyIrcDebug : ",0,str(msg))
            if msg[0] == "privmsg":
                if msg[4][0] not in ['#','&','!']:
                    if msg[3] not in self.queries:
                        self.queries.append(msg[3])
                        self._out('witemadd',"'"+msg[3]+"',0")
                        self._out('witemchg',"'"+msg[3]+"'")
                    html = self.default_format['message private window'] % (msg[3],self._escape(msg[5][:-2]))
                    data = "'"+msg[3]+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
                else:
                    if string.find(msg[5][:-2],self.nickname) == -1:
                        html = self.default_format['message public'] % (msg[3],self._escape(msg[5][:-2]))
                    else:
                        html = self.default_format['message public hilight'] % (msg[3],self._escape(msg[5][:-2]))
                    data = "'"+msg[4]+"',"+str("'<span>"+html+"</span><br>'")+",1,1" 
                self._out('witemaddtext',data)
            elif msg[0] == "nick":
                if msg[3] == self.nickname:
                    self.nickname = msg[4][:-2]
                self._out('channelsusernick',str("'"+msg[3]+"','"+msg[4][:-2]+"'"))
                html = self.default_format['nick'] % (msg[3],msg[4][:-2])
                data = "'-all',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "join":
                if msg[3] == self.nickname:
                    self._out('witemadd',"'"+msg[4][:-2]+"',1")
                    self._out('witemchg',"'"+msg[4][:-2]+"'")
                html = self.default_format['join'] % (msg[3],msg[2],msg[4][:-2])
                data = "'"+msg[4][:-2]+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
                self._out('channeladdusers',str("'"+msg[4][:-2]+"',['"+msg[3]+"']"))
            elif msg[0] == "part":
                if msg[3] != self.nickname:
                     html = self.default_format['part'] % (msg[3],msg[4],self._escape(msg[5]))
                     data = "'"+msg[4]+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
                     self._out('witemaddtext',data)
                     self._out('channeldeluser',str("'"+msg[4]+"','"+msg[3]+"'"))
            elif msg[0] == "quit":
                html = self.default_format['quit'] % (msg[3],msg[2],self._escape(msg[4][:-2]))
                data = "'-all',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
                self._out('channelsdeluser',str("'-all-','"+msg[3]+"'"))
            elif msg[0] == "ctcp":
                if msg[5] != "ACTION":
                    html = self.default_format['ctcp msg'] % (msg[3],msg[5],msg[4],self._escape(msg[6]))
                    data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                else:
                    html = self.default_format['action public own'] % (msg[3],self._escape(msg[6]))
                    html = self._escape(html)
                    data = "'"+msg[4]+"',"+str("'<span>"+html+"</span><br>'")+",1,1" 
                self._out('witemaddtext',data)
            elif msg[0] in ["startmotd","endmotd"]:
                html = self.default_format['server info'] % self._escape(msg[2][:-2])
                data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "topic":
                if msg[1] != msg[2]:
                    html = self.default_format['topic'] % (msg[3],self._escape(msg[5][:-2]))
                    data = "'"+msg[4]+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
                else:
                    html = self.default_format['reply topic'] % self._escape(msg[5][:-2])
                    data = "'"+msg[4]+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "isaway":
                html = self.default_format['reply away'] % (msg[3],self._escape(msg[4][:-2]))
                data = "'"+msg[3]+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "markedaway":
                html = self.default_format['reply nowaway']
                data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "unmarkedaway":
                html = self.default_format['reply unaway']
                data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] =="notopic":
                html = self.default_format['reply notopic']
                data = "'"+msg[4]+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "datamotd":
                data = sub(_motd_spaces_to_nbsp,"&nbsp;",msg[2][:-2])
                if data[0] == ' ':
                  data = "&nbsp;" + data[1:]  
                html = self.default_format['motd'] % self._escape(data)
                data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "init_info":
                if msg[2] == "001":
                    self.REQUEST.RESPONSE.write('<script language="JavaScript" type="text/JavaScript">parent.set_connected(1);</script>\r\n')
                if msg[2] in ['001','002','003']:
                    html = self.default_format['server info'] % msg[3][:-2]
                elif msg[2] == '004':
                    html = self.default_format['server info'] % string.join(msg[4:8],' ')
                else:
                    html = self.default_format['server info'] % string.join(msg[3],' ')
                data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "luser":
                html = self.default_format['server info'] % self._escape(msg[3][:-2])
                data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "unknown":
                html = self.default_format['server info'] % self._escape(msg[5][:-2])
                data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "datanames":
                if len(msg[3]) != 0:
                    nicks = []
                    for n in msg[3]:
                        if n[0] not in ['@','+','%']:
                            n = ' ' + n
                        nicks.append(str("'"+n+"'"))
                    self._out('channeladdusers',str("'"+msg[2]+"',["+string.join(nicks,',')+"]"))
            elif msg[0] == "notice":
                if msg[4][0] in ['#','&','!']:
                    html = self.default_format['notice public'] % (msg[3],msg[4],self._escape(msg[5][:-2]))
                    data = "'"+msg[4]+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
                elif msg[4] == self.nickname and msg[3] != '':
                    html = self.default_format['notice private'] % (msg[3],self._escape(msg[5][:-2]))
                    data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                else:
                    html = self.default_format['notice server'] % (msg[1],self._escape(msg[5][:-2]))
                    data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "mode":
                # Ugly hack... will be fixed in pyirc itself one day
                if len(msg) == 7:
                    type = ''
                    if msg[5][1] == 'o':
                        type = 'op'
                    elif msg[5][1] == 'v':
                        type = 'voice'
                    action = msg[5][0]
                    if type != '':
                        action_on_user = msg[5]+" "+msg[6]
                        html = self.default_format['mode'] % (action_on_user,msg[3])
                        data = "'"+msg[4]+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
                        self._out('witemaddtext',data)
                        self._out('channelusermode',str("'"+msg[4]+"','"+msg[6]+"','"+action+"','"+type+"'"))
            elif msg[0] == "kick":
                html = self.default_format['kick'] % (msg[3],msg[4],msg[3],'')
                data = "'"+msg[4]+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
                self._out('channeldeluser',str("'"+msg[4]+"','"+msg[5][:-2]+"'"))
            elif msg[0] == "invite":
                html = self.default_format['invite'] % (msg[3],msg[5][:-2])
                data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "topic_info":
                html = self.default_format['reply topicwhotime'] % (msg[4],strftime("%a, %d %b %Y %H:%M:%S", gmtime(int(msg[5]))))
                data = "'"+msg[3]+"',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "datawhois":
                if msg[2] == "311":
                    html = self.default_format['reply whois user'] % (msg[3],msg[4],msg[5],msg[7][:-2])
                elif msg[2] == "312":
                    html = self.default_format['reply whois server'] % (msg[4],msg[5][:-2])
                elif msg[2] == "318":
                    html = self.default_format['reply end whois'] % msg[4][:-2]
                elif msg[2] == "319":
                    html = self.default_format['reply whois channel'] % msg[4][:-2]
                data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
            elif msg[0] == "error":
                if msg[2] == "433":
                    html = self.default_format['error nickinuse'] % msg[3]
                    data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                elif msg[2] == "401":
                    html = self.default_format['error nosuchnick'] % msg[3]
                    data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                else:
                    html = self.default_format['error'] % str(msg[3]+": "+self._escape(msg[4][:-2]))
                    data = "'Status',"+str("'<span>"+html+"</span><br>'")+",1,1"
                self._out('witemaddtext',data)
                
InitializeClass(ZPyIrcConnexion)
